-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftFrame")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Frame"] = true,
	["Frame options"] = true,
	["Width"] = true,
	["Sets the width of the frame"] = true,
	["Height"] = true,
	["Sets the height of the frame"] = true,

	["Packing Width"] = true,
	["Sets the minimum horizontal distance between frames"] = true,
	["Packing Height"] = true,
	["Sets the minimum vertical distance between frames"] = true,

	["Background Color"] = true,
	["Sets the background color of the nameplate"] = true,
	["Border Color"] = true,
	["Sets the border color of the nameplate"] = true,
	["Border Style"] = true,
	["Sets the style of the nameplate border"] = true,
	
	["NONE"] = "None", 
	["STANDARD"] = "Standard", 
	["SQUARE"] = "Square",
} end)

-----------------------------------------------------------------------------

local borderData =
{
	["NONE"] 		= { texture = "", size = 0, inset = 0 },
	["STANDARD"] 	= { texture = "Interface\\Tooltips\\UI-Tooltip-Border", size = 16, inset = 5 },
	["SQUARE"]	 	= { texture = "Interface\\DialogFrame\\UI-DialogBox-Border", size = 16, inset = 5 },
}

-----------------------------------------------------------------------------

local AloftFrame = Aloft:NewModule("Frame", "AceEvent-2.0")
AloftFrame.db = Aloft:AcquireDBNamespace("frame")
AloftFrame.healthBarDB = Aloft:AcquireDBNamespace("healthBar")
Aloft:RegisterDefaults("frame", "profile", {
	borderStyle		= "NONE",
	borderColor		= { 1, 1, 1, 1 },
	backgroundColor	= { 0, 0, 0, 0 },
	width			= 10,
	height			= 10,
	packingWidth	= 10,
	packingHeight	= 10,
})

Aloft.Options.args.frame =
{
	type = 'group',
	name = L["Frame"],
	desc = L["Frame options"],
	disabled = function() return not Aloft:IsActive() end,
	args =
	{
		width =
		{
			type = 'range',
			name = L["Width"],
			desc = L["Sets the width of the frame"],
			min = 0,
			max = 100,
			step = 1,
			get = function() return AloftFrame.db.profile.width end,
			set = function(v) if AloftFrame.db.profile.width ~= v then AloftFrame.db.profile.width = v AloftFrame.db.profile.packingWidth = v AloftFrame:UpdateAll() end end,
		},
		height =
		{
			type = 'range',
			name = L["Height"],
			desc = L["Sets the height of the frame"],
			min = 0,
			max = 100,
			step = 1,
			get = function() return AloftFrame.db.profile.height end,
			set = function(v) if AloftFrame.db.profile.height ~= v then AloftFrame.db.profile.height = v AloftFrame.db.profile.packingHeight = v AloftFrame:UpdateAll() end end,
		},
		packingWidth =
		{
			type = 'range',
			name = L["Packing Width"],
			desc = L["Sets the minimum horizontal distance between frames"],
			min = 0,
			max = 100,
			step = 1,
			get = function() return AloftFrame.db.profile.packingWidth end,
			set = function(v) AloftFrame.db.profile.packingWidth = v AloftFrame:UpdateAll() end,
		},
		packingHeight =
		{
			type = 'range',
			name = L["Packing Height"],
			desc = L["Sets the minimum vertical distance between frames"],
			min = 0,
			max = 100,
			step = 1,
			get = function() return AloftFrame.db.profile.packingHeight end,
			set = function(v) AloftFrame.db.profile.packingHeight = v AloftFrame:UpdateAll() end,
		},
		backgroundColor =
		{
			type = 'color',
			name = L["Background Color"],
			desc = L["Sets the background color of the nameplate"],
			get = function() return unpack(AloftFrame.db.profile.backgroundColor) end,
			set = function(r, g, b, a) AloftFrame.db.profile.backgroundColor = { r, g, b, a } AloftFrame:UpdateAll() end,
			hasAlpha = true,
		},
		borderColor =
		{
			type = 'color',
			name = L["Border Color"],
			desc = L["Sets the border color of the nameplate"],
			get = function() return unpack(AloftFrame.db.profile.borderColor) end,
			set = function(r, g, b, a) AloftFrame.db.profile.borderColor = { r, g, b, a } AloftFrame:UpdateAll() end,
			hasAlpha = true,
		},
		borderStyle =
		{
			type = 'text',
			name = L["Border Style"],
			desc = L["Sets the style of the nameplate border"],
			get = function() return AloftFrame.db.profile.borderStyle end,
			set = function(v) AloftFrame.db.profile.borderStyle = v AloftFrame:UpdateAll() end,
			validate = { ["NONE"] = L["NONE"], ["STANDARD"] = L["STANDARD"], ["SQUARE"] = L["SQUARE"] }
		},
	},
}

-----------------------------------------------------------------------------

function AloftFrame:UpdateAll()
	for aloftData in Aloft:IterateNameplates() do
		self:SetupFrame(aloftData)
	end
	
	for aloftData in Aloft:IterateVisibleNameplates() do
		self:OnNameplateShow(aloftData)
	end
end

-----------------------------------------------------------------------------

function AloftFrame:OnEnable()
	self:RegisterEvent("Aloft:SetupFrame", "SetupFrame")
	self:RegisterEvent("Aloft:OnNameplateShow", "OnNameplateShow")
	self:UpdateAll()
end

function AloftFrame:OnDisable()
	self:UnregisterAllEvents()

	for aloftData in Aloft:IterateNameplates() do
		local backdropFrame = aloftData.backdropFrame
		if backdropFrame then
			backdropFrame:Hide()
		end
		aloftData.nameplateFrame:SetFrameLevel(1)
		aloftData.healthBar:SetFrameLevel(0)
		aloftData.castBar:SetFrameLevel(0)
	end
end

function AloftFrame:SetupFrame(aloftData)
	--
	-- Using the inbuilt frame works fine, until a nameplate shows during combat
	--
	-- The game likes to automatically resize the frame when a frame is shown, 
	-- but we can't change it back OnShow because of in-combat lockdown
	--
	-- We also can't use SetFrameLevel during combat
	--
	-- Approach:
	--  If backdrop or border are required, then:
	--   1. Create a backdrop frame
	--   2. Create a region frame
	--   3. Adjust healthbar/castbar
	--	
	local profile = self.db.profile

	local backdropFrame = aloftData.backdropFrame
	
	if profile.borderStyle == "NONE" and profile.backgroundColor[4] == 0 then
		if backdropFrame then
			backdropFrame:Hide()
		end
		aloftData.healthBar:SetFrameLevel(1)
		aloftData.castBar:SetFrameLevel(1)
		return
	end
	
	if not backdropFrame then
		local UIFrame = CreateFrame("Frame", nil, aloftData.nameplateFrame)
		UIFrame:SetAllPoints(aloftData.nameplateFrame)
		UIFrame:SetFrameLevel(3)
		aloftData.UIFrame = UIFrame

		backdropFrame = CreateFrame("Frame", nil, UIFrame)
		backdropFrame:SetPoint("CENTER", aloftData.nameplateFrame, "CENTER")
		aloftData.backdropFrame = backdropFrame
	end

	local data = borderData[profile.borderStyle]
	local healthBarProfile = self.healthBarDB.profile

	backdropFrame:SetBackdrop({
								bgFile = "Interface\\ChatFrame\\ChatFrameBackground", tile = true, tileSize = 16,
								edgeFile = data.texture, edgeSize = data.size,
								insets = {left = data.inset, right = data.inset, top = data.inset, bottom = data.inset},
							  })

	backdropFrame:SetBackdropColor(unpack(profile.backgroundColor))
	backdropFrame:SetBackdropBorderColor(unpack(profile.borderColor))
	backdropFrame:SetWidth(profile.width+healthBarProfile.width)
	backdropFrame:SetHeight(profile.height+healthBarProfile.height)
	backdropFrame:SetFrameLevel(1)
	backdropFrame:Show(1)

	aloftData.healthBar:SetFrameLevel(2)
	aloftData.castBar:SetFrameLevel(2)
end

function AloftFrame:AdoptRegions(aloftData, ...)
	for i = 1, select('#',...) do
		local region = select(i, ...)
		region:SetParent(aloftData.UIFrame)
	end
end

function AloftFrame:AdoptChildren(aloftData, ...)
	local UIFrame = aloftData.UIFrame

	for i = 1, select('#',...) do
		local child = select(i, ...)
		if child ~= UIFrame then
			local frameLevel = child:GetFrameLevel()
			child:SetParent(UIFrame)
			child:SetFrameLevel(frameLevel)
			child:SetAlpha(UIFrame:GetAlpha())
		end
	end
end

function AloftFrame:OnNameplateShow(aloftData)
	local profile = self.db.profile
	local healthBarProfile = self.healthBarDB.profile

	local nameplateFrame = aloftData.nameplateFrame

	if aloftData.backdropFrame then
		self:AdoptRegions(aloftData, nameplateFrame:AloftGetRegions())
		self:AdoptChildren(aloftData, nameplateFrame:AloftGetChildren())
	end

	nameplateFrame:SetWidth(profile.packingWidth+healthBarProfile.width)
	nameplateFrame:SetHeight(profile.packingHeight+healthBarProfile.height)
end

-----------------------------------------------------------------------------
